@@ -29,12 +29,11 @@ int arch_load_dynsymtab_noplt(struct uftrace_symtab *dsymtab, struct uftrace_elf
 
 	memset(dsymtab, 0, sizeof(*dsymtab));
 
-	/* assumes there's only one RELA section (rela.dyn) for no-plt binary */
 	elf_for_each_shdr(elf, &sec_iter) {
-		if (sec_iter.shdr.sh_type == SHT_RELA) {
+		if (strcmp(elf_get_name(elf, &sec_iter, sec_iter.shdr.sh_name), ".rela.dyn") == 0) {
 			memcpy(&rel_iter, &sec_iter, sizeof(sec_iter));
-			pr_dbg2("found RELA section: %s\n",
-				elf_get_name(elf, &sec_iter, sec_iter.shdr.sh_name));
+			pr_dbg2("found rela.dyn section with %ld entry.\n",
+				sec_iter.shdr.sh_entsize);
 
 			reloc_start = rel_iter.shdr.sh_addr + offset;
 			reloc_entsize = rel_iter.shdr.sh_entsize;
@@ -93,6 +92,7 @@ int arch_load_dynsymtab_noplt(struct uftrace_symtab *dsymtab, struct uftrace_elf
 		pr_dbg3("[%zd] %c %lx + %-5u %s\n", dsymtab->nr_sym, sym->type, sym->addr,
 			sym->size, sym->name);
 	}
+	sort_dynsymtab(dsymtab);
 
 	return dsymtab->nr_sym;
 }
