@@ -4,7 +4,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-int main()
+int main(void)
 {
 	cs_insn insn;
 	printf("size: %zu\n", sizeof(insn));
