@@ -182,11 +182,11 @@ static const char *graph_field_names[NUM_GRAPH_FIELD] = {
 	"ADDRESS",
 };
 
-#define NUM_REPORT_FIELD 10
+#define NUM_REPORT_FIELD 12
 
 static const char *report_field_names[NUM_REPORT_FIELD] = {
-	"TOTAL TIME", "TOTAL AVG", "TOTAL MIN", "TOTAL MAX", "SELF TIME",
-	"SELF AVG",   "SELF MIN",  "SELF MAX",	"CALL",	     "SIZE",
+	"TOTAL TIME", "TOTAL AVG", "TOTAL MIN", "TOTAL MAX", "SELF TIME",  "SELF AVG",
+	"SELF MIN",   "SELF MAX",  "CALL",	"SIZE",	     "TOTAL STDV", "SELF STDV",
 };
 
 static const char *field_help[] = {
@@ -204,8 +204,8 @@ enum tui_mode {
 };
 
 static char *report_sort_key[] = {
-	OPT_SORT_KEYS, "total_avg", "total_min", "total_max", "self",
-	"self_avg",    "self_min",  "self_max",	 "call",      "size",
+	OPT_SORT_KEYS, "total_avg", "total_min", "total_max", "self",	    "self_avg",
+	"self_min",    "self_max",  "call",	 "size",      "total_stdv", "self_stdv",
 };
 
 static char *selected_report_sort_key[NUM_REPORT_FIELD];
@@ -350,20 +350,26 @@ static void print_report_##_func(struct field_data *fd)
 {                                                                                                  \
 	struct uftrace_report_node *node = fd->arg;                                                \
 	uint64_t d = node->_field;                                                                 \
-	printw("  ");                                                                              \
 	print_time(d);                                                                             \
 }                                                                                                  \
-REPORT_FIELD_STRUCT(_id, _name, _func, _header, 11)
+REPORT_FIELD_STRUCT(_id, _name, _func, _header, 10)
+
+#define REPORT_FIELD_PERCENTAGE(_id, _name, _field, _func, _header)                                \
+static void print_report_##_func(struct field_data *fd)                                            \
+{                                                                                                  \
+	struct uftrace_report_node *node = fd->arg;                                                \
+	printw("%9.2f%%", node->_field);                                                           \
+}                                                                                                  \
+REPORT_FIELD_STRUCT(_id, _name, _func, _header, 10)
 
 #define REPORT_FIELD_UINT(_id, _name, _field, _func, _header)                                      \
 static void print_report_##_func(struct field_data *fd)                                            \
 {                                                                                                  \
 	struct uftrace_report_node *node = fd->arg;                                                \
 	uint64_t d = node->_field;                                                                 \
-	printw("  ");                                                                              \
 	printw("%10"PRIu64 "", d);                                                                 \
 }                                                                                                  \
-REPORT_FIELD_STRUCT(_id, _name, _func, _header, 11)
+REPORT_FIELD_STRUCT(_id, _name, _func, _header, 10)
 
 REPORT_FIELD_TIME(REPORT_F_TOTAL_TIME, total, total.sum, total, "TOTAL TIME");
 REPORT_FIELD_TIME(REPORT_F_TOTAL_TIME_AVG, total-avg, total.avg, total_avg, "TOTAL AVG");
@@ -375,14 +381,16 @@ REPORT_FIELD_TIME(REPORT_F_SELF_TIME_MIN, self-min, self.min, self_min, "SELF MI
 REPORT_FIELD_TIME(REPORT_F_SELF_TIME_MAX, self-max, self.max, self_max, "SELF MAX");
 REPORT_FIELD_UINT(REPORT_F_CALL, call, call, call, "CALL");
 REPORT_FIELD_UINT(REPORT_F_SIZE, size, size, size, "SIZE");
+REPORT_FIELD_PERCENTAGE(REPORT_F_TOTAL_TIME_STDV, total-stdv, total.stdv, total_stdv, "TOTAL STDV");
+REPORT_FIELD_PERCENTAGE(REPORT_F_SELF_TIME_STDV, self-stdv, self.stdv, self_stdv, "SELF STDV");
 
 /* clang-format on */
 
 static struct display_field *report_field_table[] = {
-	&report_field_total,	 &report_field_total_avg, &report_field_total_min,
-	&report_field_total_max, &report_field_self,	  &report_field_self_avg,
-	&report_field_self_min,	 &report_field_self_max,  &report_field_call,
-	&report_field_size,
+	&report_field_total,	 &report_field_total_avg,  &report_field_total_min,
+	&report_field_total_max, &report_field_self,	   &report_field_self_avg,
+	&report_field_self_min,	 &report_field_self_max,   &report_field_call,
+	&report_field_size,	 &report_field_total_stdv, &report_field_self_stdv,
 };
 
 static void setup_default_graph_field(struct list_head *fields, struct uftrace_opts *opts,
@@ -732,7 +740,7 @@ static struct tui_graph *tui_graph_init(struct uftrace_opts *opts)
 	list_for_each_entry(graph, &tui_graph_list, list) {
 		/* top (root) is an artificial node, fill the info */
 		top = &graph->ug.root;
-		top->name = basename(graph->ug.sess->exename);
+		top->name = (char *)uftrace_basename(graph->ug.sess->exename);
 		top->nr_calls = 1;
 
 		list_for_each_entry(node, &graph->ug.root.head, list) {
@@ -1545,13 +1553,13 @@ static void win_header_report(struct tui_window *win, struct uftrace_data *handl
 	buf = p = xmalloc(w + 1);
 
 	list_for_each_entry(field, &report_output_fields, list) {
-		char header[field->length + 1];
+		char header[field->length + 2];
 
 		header[0] = '\0';
 		if (i == curr_sort_key)
 			strcpy(header, "*");
 		strcat(header, field->header);
-		c = snprintf(p, w, " %*s", field->length, header);
+		c = snprintf(p, w, " %*s", field->length + 1, header);
 		p += c;
 		w -= c;
 		i++;
@@ -1604,8 +1612,9 @@ static void win_display_report(struct tui_window *win, void *node)
 	int w = 2;
 
 	list_for_each_entry(field, &report_output_fields, list) {
+		printw("  ");
 		field->print(&fd);
-		w += field->length + 1;
+		w += field->length + 2;
 	}
 
 	printw("  ");
@@ -1871,7 +1880,7 @@ static void win_display_session(struct tui_window *win, void *node)
 		curr_sess = partial_graph.ug.sess;
 		get_current_graph(node, &count);
 		print_buf(" %c  %s #%d: %s", s == curr_sess ? 'G' : ' ', "call Graph for session",
-			  count, basename(s->exename));
+			  count, uftrace_basename(s->exename));
 		break;
 	}
 
