@@ -93,9 +93,19 @@ struct mcount_event {
 
 #define MAX_EVENT 4
 
-enum mcount_watch_item {
+enum mcount_watch_kind {
 	MCOUNT_WATCH_NONE = 0,
 	MCOUNT_WATCH_CPU = (1 << 0),
+	MCOUNT_WATCH_VAR = (1 << 1),
+};
+
+struct mcount_watchpoint_item {
+	struct list_head list;
+	unsigned long addr;
+	unsigned short kind;
+	unsigned char size;
+	unsigned char inited;
+	unsigned char data[];
 };
 
 struct mcount_watchpoint {
@@ -104,6 +114,7 @@ struct mcount_watchpoint {
 	int cpu;
 
 	/* global watch points */
+	struct list_head list;
 };
 
 #ifndef DISABLE_MCOUNT_FILTER
@@ -185,7 +196,7 @@ extern uint64_t mcount_threshold; /* nsec */
 extern unsigned mcount_minsize;
 extern pthread_key_t mtd_key;
 extern int shmem_bufsize;
-extern int pfd;
+extern int mcount_pfd;
 extern int mcount_depth;
 extern char *mcount_exename;
 extern int page_size_in_kb;
@@ -225,6 +236,11 @@ static inline void mcount_watch_setup(struct mcount_thread_data *mtdp)
 static inline void mcount_watch_release(struct mcount_thread_data *mtdp)
 {
 }
+static inline struct uftrace_triggers_info *
+mcount_trigger_init(struct uftrace_filter_setting *filter_setting)
+{
+	return NULL;
+}
 #endif /* DISABLE_MCOUNT_FILTER */
 
 static inline uint64_t mcount_gettime(void)
@@ -306,11 +322,11 @@ extern void uftrace_send_message(int type, void *data, size_t len);
 extern void build_debug_domain(char *dbg_domain_str);
 
 extern void mcount_rstack_restore(struct mcount_thread_data *mtdp);
-extern void mcount_rstack_reset(struct mcount_thread_data *mtdp);
-extern void mcount_rstack_reset_exception(struct mcount_thread_data *mtdp,
-					  unsigned long frame_addr);
+extern void mcount_rstack_rehook(struct mcount_thread_data *mtdp);
+extern void mcount_rstack_rehook_exception(struct mcount_thread_data *mtdp,
+					   unsigned long frame_addr);
 extern void mcount_auto_restore(struct mcount_thread_data *mtdp);
-extern void mcount_auto_reset(struct mcount_thread_data *mtdp);
+extern void mcount_auto_rehook(struct mcount_thread_data *mtdp);
 extern bool mcount_rstack_has_plthook(struct mcount_thread_data *mtdp);
 
 extern void prepare_shmem_buffer(struct mcount_thread_data *mtdp);
@@ -423,12 +439,14 @@ extern void save_argument(struct mcount_thread_data *mtdp, struct mcount_ret_sta
 void save_retval(struct mcount_thread_data *mtdp, struct mcount_ret_stack *rstack, long *retval);
 void save_trigger_read(struct mcount_thread_data *mtdp, struct mcount_ret_stack *rstack,
 		       enum trigger_read_type type, bool diff);
+struct uftrace_triggers_info *mcount_trigger_init(struct uftrace_filter_setting *filter_setting);
 #endif /* DISABLE_MCOUNT_FILTER */
 
 bool check_mem_region(struct mcount_arg_context *ctx, unsigned long addr);
 
 void save_watchpoint(struct mcount_thread_data *mtdp, struct mcount_ret_stack *rstack,
 		     unsigned long watchpoints);
+bool mcount_watch_update(unsigned long addr, void *data, int size);
 
 struct mcount_event_info {
 	char *module;
@@ -460,4 +478,6 @@ bool mcount_is_main_executable(const char *filename, const char *exename);
 int agent_spawn(void);
 int agent_kill(void);
 
+void swap_triggers(struct uftrace_triggers_info **old, struct uftrace_triggers_info *new);
+
 #endif /* UFTRACE_MCOUNT_INTERNAL_H */
