@@ -32,6 +32,21 @@
 #include "utils/utils.h"
 #include "version.h"
 
+/*
+ * mcount global variables.
+ *
+ * These are to control various features in the libmcount.
+ * They are set during initialization (mcount_startup) which I believe, runs in
+ * a single thread.  After that multiple threads (mostly) read the value so it's
+ * not protected by lock or something.  So special care needs to be taken if you
+ * want to change it at runtime (like in the agent).
+ *
+ * Some are marked as 'maybe unused' because they are only used when filter
+ * functions are implemented.  Note that libmcount is built with different
+ * settings like -fast and -single to be more efficient in some situation like
+ * when no filter is specified in the command line and/or single-thread only.
+ */
+
 /* time filter in nsec */
 uint64_t mcount_threshold;
 
@@ -55,11 +70,17 @@ unsigned long mcount_global_flags = MCOUNT_GFL_SETUP;
 /* TSD key to save mtd below */
 pthread_key_t mtd_key = (pthread_key_t)-1;
 
-/* thread local data to trace function execution */
+/*
+ * A thread local data to trace function execution.
+ * While this is itself TLS so ok to by accessed safely by each thread,
+ * mcount routines use TSD APIs to access it for performance reason.
+ * Also TSD provides destructor so it can release the resources when the thread
+ * exits.
+ */
 TLS struct mcount_thread_data mtd;
 
 /* pipe file descriptor to communite to uftrace */
-int pfd = -1;
+int mcount_pfd = -1;
 
 /* maximum depth of mcount rstack */
 static int mcount_rstack_max = MCOUNT_RSTACK_MAX;
@@ -102,7 +123,14 @@ __weak void dynamic_return(void)
 {
 }
 
+/* list of watch points (of global variables) */
+static LIST_HEAD(mcount_watch_list);
+
 #ifdef DISABLE_MCOUNT_FILTER
+/*
+ * These functions are only the FAST version of libmcount libraries which don't
+ * implement filters (other than time and size filters).
+ */
 
 static void mcount_filter_init(struct uftrace_filter_setting *filter_setting, bool force)
 {
@@ -121,7 +149,14 @@ static void mcount_filter_finish(void)
 	finish_debug_info(&mcount_sym_info);
 }
 
+static void mcount_watch_finish(void)
+{
+}
+
 #else
+/*
+ * Here goes the regular libmcount's filter and trigger functions.
+ */
 
 /* be careful: this can be called from signal handler */
 static void mcount_finish_trigger(void)
@@ -356,24 +391,19 @@ static void mcount_signal_finish(void)
 	}
 }
 
-static void mcount_filter_init(struct uftrace_filter_setting *filter_setting, bool force)
+struct uftrace_triggers_info *mcount_trigger_init(struct uftrace_filter_setting *filter_setting)
 {
+	struct uftrace_triggers_info *triggers;
 	char *filter_str = getenv("UFTRACE_FILTER");
 	char *trigger_str = getenv("UFTRACE_TRIGGER");
 	char *argument_str = getenv("UFTRACE_ARGUMENT");
 	char *retval_str = getenv("UFTRACE_RETVAL");
 	char *autoargs_str = getenv("UFTRACE_AUTO_ARGS");
+	char *patch_str = getenv("UFTRACE_PATCH");
 	char *caller_str = getenv("UFTRACE_CALLER");
 	char *loc_str = getenv("UFTRACE_LOCATION");
 	bool needs_debug_info = false;
 
-	filter_setting->lp64 = host_is_lp64();
-	filter_setting->arch = host_cpu_arch();
-
-	load_module_symtabs(&mcount_sym_info);
-
-	mcount_signal_init(getenv("UFTRACE_SIGNAL"), filter_setting);
-
 	/* setup auto-args only if argument/return value is used */
 	if (argument_str || retval_str || autoargs_str ||
 	    (trigger_str && (strstr(trigger_str, "arg") || strstr(trigger_str, "retval")))) {
@@ -387,27 +417,29 @@ static void mcount_filter_init(struct uftrace_filter_setting *filter_setting, bo
 	/* use debug info if available */
 	if (needs_debug_info) {
 		prepare_debug_info(&mcount_sym_info, filter_setting->ptype, argument_str,
-				   retval_str, !!autoargs_str, force);
+				   retval_str, !!autoargs_str, !!patch_str);
 		save_debug_info(&mcount_sym_info, mcount_sym_info.dirname);
 	}
 
-	mcount_triggers = xmalloc(sizeof(*mcount_triggers));
-	memset(mcount_triggers, 0, sizeof(*mcount_triggers));
-	mcount_triggers->root = RB_ROOT;
-	uftrace_setup_filter(filter_str, &mcount_sym_info, mcount_triggers, filter_setting);
-	uftrace_setup_trigger(trigger_str, &mcount_sym_info, mcount_triggers, filter_setting);
-	uftrace_setup_argument(argument_str, &mcount_sym_info, mcount_triggers, filter_setting);
-	uftrace_setup_retval(retval_str, &mcount_sym_info, mcount_triggers, filter_setting);
+	if (!filter_str && !trigger_str && !argument_str && !retval_str && !autoargs_str &&
+	    !caller_str && !loc_str)
+		return NULL;
 
-	if (needs_debug_info) {
-		uftrace_setup_loc_filter(loc_str, &mcount_sym_info, mcount_triggers,
-					 filter_setting);
-	}
+	triggers = xzalloc(sizeof(*triggers));
+	triggers->root = RB_ROOT;
 
-	if (caller_str) {
-		uftrace_setup_caller_filter(caller_str, &mcount_sym_info, mcount_triggers,
-					    filter_setting);
-	}
+	filter_setting->auto_args = false;
+
+	uftrace_setup_filter(filter_str, &mcount_sym_info, triggers, filter_setting);
+	uftrace_setup_trigger(trigger_str, &mcount_sym_info, triggers, filter_setting);
+	uftrace_setup_argument(argument_str, &mcount_sym_info, triggers, filter_setting);
+	uftrace_setup_retval(retval_str, &mcount_sym_info, triggers, filter_setting);
+
+	if (needs_debug_info)
+		uftrace_setup_loc_filter(loc_str, &mcount_sym_info, triggers, filter_setting);
+
+	if (caller_str)
+		uftrace_setup_caller_filter(caller_str, &mcount_sym_info, triggers, filter_setting);
 
 	if (autoargs_str) {
 		char *autoarg = ".";
@@ -418,8 +450,27 @@ static void mcount_filter_init(struct uftrace_filter_setting *filter_setting, bo
 
 		filter_setting->auto_args = true;
 
-		uftrace_setup_argument(autoarg, &mcount_sym_info, mcount_triggers, filter_setting);
-		uftrace_setup_retval(autoret, &mcount_sym_info, mcount_triggers, filter_setting);
+		uftrace_setup_argument(autoarg, &mcount_sym_info, triggers, filter_setting);
+		uftrace_setup_retval(autoret, &mcount_sym_info, triggers, filter_setting);
+	}
+
+	return triggers;
+}
+
+static void mcount_filter_init(struct uftrace_filter_setting *filter_setting, bool force)
+{
+	filter_setting->lp64 = host_is_lp64();
+	filter_setting->arch = host_cpu_arch();
+
+	load_module_symtabs(&mcount_sym_info);
+
+	mcount_signal_init(getenv("UFTRACE_SIGNAL"), filter_setting);
+
+	mcount_triggers = mcount_trigger_init(filter_setting);
+	if (mcount_triggers == NULL) {
+		/* make sure it has the root of triggers */
+		mcount_triggers = xzalloc(sizeof(*mcount_triggers));
+		mcount_triggers->root = RB_ROOT;
 	}
 
 	if (getenv("UFTRACE_DEPTH"))
@@ -471,23 +522,113 @@ static void mcount_watch_init(void)
 	strv_split(&watch, watch_str, ";");
 
 	strv_for_each(&watch, str, i) {
-		if (!strcasecmp(str, "cpu"))
-			mcount_watchpoints = MCOUNT_WATCH_CPU;
+		if (!strcasecmp(str, "cpu")) {
+			mcount_watchpoints |= MCOUNT_WATCH_CPU;
+			continue;
+		}
+
+		if (!strncasecmp(str, "var:", 4)) {
+			struct mcount_watchpoint_item *w;
+			struct uftrace_mmap *map = mcount_sym_info.exec_map;
+			struct uftrace_symbol *sym;
+
+			w = xmalloc(sizeof(*w));
+			sym = find_symname(&map->mod->symtab, str + 4);
+			if (sym == NULL) {
+				pr_dbg("ignore watchpoint for %s\n", str);
+				free(w);
+				continue;
+			}
+
+			w->kind = MCOUNT_WATCH_VAR;
+			w->addr = map->start + sym->addr;
+			w->size = sym->size;
+			if (w->size > 8) {
+				pr_dbg("symbol is too big, ignored... %s\n", str);
+				free(w);
+				continue;
+			}
+
+			list_add_tail(&w->list, &mcount_watch_list);
+
+			mcount_watchpoints |= MCOUNT_WATCH_VAR;
+			continue;
+		}
 	}
 	strv_free(&watch);
 }
 
+static void mcount_watch_finish(void)
+{
+	struct mcount_watchpoint_item *w;
+
+	while (!list_empty(&mcount_watch_list)) {
+		w = list_first_entry(&mcount_watch_list, typeof(*w), list);
+		list_del(&w->list);
+		free(w);
+	}
+}
+
+bool mcount_watch_update(unsigned long addr, void *data, int size)
+{
+	static pthread_mutex_t watch_mutex = PTHREAD_MUTEX_INITIALIZER;
+	struct mcount_watchpoint_item *w;
+	bool updated = false;
+
+	pthread_mutex_lock(&watch_mutex);
+	list_for_each_entry(w, &mcount_watch_list, list) {
+		if (w->addr != addr)
+			continue;
+
+		/* someone already updated for us? */
+		if (w->inited && !memcmp(data, w->data, size))
+			break;
+
+		mcount_memcpy1(w->data, data, size);
+		w->inited = true;
+		updated = true;
+		break;
+	}
+	pthread_mutex_unlock(&watch_mutex);
+
+	return updated;
+}
+
 static void mcount_watch_setup(struct mcount_thread_data *mtdp)
 {
+	struct mcount_watchpoint_item *pos, *w;
+
 	mtdp->watch.cpu = -1;
+	INIT_LIST_HEAD(&mtdp->watch.list);
+
+	/* each thread gets a copy of the global watch items */
+	list_for_each_entry(pos, &mcount_watch_list, list) {
+		w = xmalloc(sizeof(*w) + pos->size);
+
+		memcpy(w, pos, sizeof(*w));
+		memcpy(w->data, (void *)w->addr, w->size);
+
+		list_add_tail(&w->list, &mtdp->watch.list);
+	}
 }
 
 static void mcount_watch_release(struct mcount_thread_data *mtdp)
 {
+	struct mcount_watchpoint_item *w;
+
+	while (!list_empty(&mtdp->watch.list)) {
+		w = list_first_entry(&mtdp->watch.list, typeof(*w), list);
+		list_del(&w->list);
+		free(w);
+	}
 }
 
 #endif /* DISABLE_MCOUNT_FILTER */
 
+/*
+ * These are common routines used in every libmcount libraries.
+ */
+
 static void send_session_msg(struct mcount_thread_data *mtdp, const char *sess_id)
 {
 	struct uftrace_msg_sess sess = {
@@ -519,14 +660,14 @@ static void send_session_msg(struct mcount_thread_data *mtdp, const char *sess_i
 	};
 	int len = sizeof(msg) + msg.len;
 
-	if (pfd < 0)
+	if (mcount_pfd < 0)
 		return;
 
 	mcount_memcpy4(sess.sid, sess_id, sizeof(sess.sid));
 
-	if (writev(pfd, iov, 3) != len) {
+	if (writev(mcount_pfd, iov, 3) != len) {
 		if (!mcount_should_stop())
-			pr_err("write tid info failed");
+			pr_err("send session msg failed");
 	}
 }
 
@@ -547,9 +688,9 @@ static void mcount_trace_finish(bool send_msg)
 	if (send_msg)
 		uftrace_send_message(UFTRACE_MSG_FINISH, NULL, 0);
 
-	if (pfd != -1) {
-		close(pfd);
-		pfd = -1;
+	if (mcount_pfd != -1) {
+		close(mcount_pfd);
+		mcount_pfd = -1;
 	}
 
 	trace_finished = true;
@@ -826,6 +967,10 @@ static bool mcount_check_rstack(struct mcount_thread_data *mtdp)
 }
 
 #ifndef DISABLE_MCOUNT_FILTER
+/*
+ * Again, this implements filter functionality used in !fast versions.
+ */
+
 extern void *get_argbuf(struct mcount_thread_data *, struct mcount_ret_stack *);
 
 /**
@@ -1108,7 +1253,7 @@ void mcount_entry_filter_record(struct mcount_thread_data *mtdp, struct mcount_r
 #define FLAGS_TO_CHECK (TRIGGER_FL_RECOVER | TRIGGER_FL_TRACE_ON | TRIGGER_FL_TRACE_OFF)
 
 		if (tr->flags & FLAGS_TO_CHECK) {
-			if (tr->flags & TRIGGER_FL_RECOVER) {
+			if ((tr->flags & TRIGGER_FL_RECOVER) && !mcount_estimate_return) {
 				mcount_rstack_restore(mtdp);
 				*rstack->parent_loc = mcount_return_fn;
 				rstack->flags |= MCOUNT_FL_RECOVER;
@@ -1153,7 +1298,7 @@ void mcount_exit_filter_record(struct mcount_thread_data *mtdp, struct mcount_re
 			mtdp->filter.out_count--;
 
 		if (rstack->flags & MCOUNT_FL_RECOVER)
-			mcount_rstack_reset(mtdp);
+			mcount_rstack_rehook(mtdp);
 	}
 
 #undef FLAGS_TO_CHECK
@@ -1167,8 +1312,16 @@ void mcount_exit_filter_record(struct mcount_thread_data *mtdp, struct mcount_re
 		if (!mcount_enabled)
 			return;
 
-		if (!(rstack->flags & MCOUNT_FL_RETVAL))
+		if (rstack->flags & MCOUNT_FL_RETVAL) {
+			struct uftrace_trigger tr;
+
+			/* update args as trigger might be updated due to dlopen() */
+			uftrace_match_filter(rstack->child_ip, &mcount_triggers->root, &tr);
+			rstack->pargs = tr.pargs;
+		}
+		else {
 			retval = NULL;
+		}
 
 		if (rstack->flags & MCOUNT_FL_READ) {
 			struct uftrace_trigger tr;
@@ -1216,6 +1369,10 @@ void mcount_exit_filter_record(struct mcount_thread_data *mtdp, struct mcount_re
 }
 
 #else /* DISABLE_MCOUNT_FILTER */
+/*
+ * Here fast versions don't implement filters.
+ */
+
 enum filter_result mcount_entry_filter_check(struct mcount_thread_data *mtdp, unsigned long child,
 					     struct uftrace_trigger *tr)
 {
@@ -1355,7 +1512,7 @@ static int __mcount_entry(unsigned long *parent_loc, unsigned long child, struct
 		if (frame_addr < (unsigned long)parent_loc)
 			frame_addr = (unsigned long)(parent_loc - 1);
 
-		mcount_rstack_reset_exception(mtdp, frame_addr);
+		mcount_rstack_rehook_exception(mtdp, frame_addr);
 		mtdp->in_exception = false;
 	}
 
@@ -1429,7 +1586,7 @@ static unsigned long __mcount_exit(long *retval)
 
 	/* re-hijack return address of parent */
 	if (mcount_auto_recover)
-		mcount_auto_reset(mtdp);
+		mcount_auto_rehook(mtdp);
 
 	__mcount_unguard_recursion(mtdp);
 
@@ -1493,7 +1650,7 @@ static int __cygprof_entry(unsigned long parent, unsigned long child)
 		if (frame_addr < (unsigned long)frame_ptr)
 			frame_addr = (unsigned long)frame_ptr;
 
-		mcount_rstack_reset_exception(mtdp, frame_addr);
+		mcount_rstack_rehook_exception(mtdp, frame_addr);
 		mtdp->in_exception = false;
 	}
 
@@ -1640,7 +1797,7 @@ static void _xray_entry(unsigned long parent, unsigned long child, struct mcount
 		if (frame_addr < (unsigned long)frame_ptr)
 			frame_addr = (unsigned long)frame_ptr;
 
-		mcount_rstack_reset_exception(mtdp, frame_addr);
+		mcount_rstack_rehook_exception(mtdp, frame_addr);
 		mtdp->in_exception = false;
 	}
 
@@ -1886,7 +2043,7 @@ static __used void mcount_startup(void)
 		dirname = UFTRACE_DIR_NAME;
 
 	xasprintf(&channel, "%s/%s", dirname, ".channel");
-	pfd = open(channel, O_WRONLY);
+	mcount_pfd = open(channel, O_WRONLY);
 	free(channel);
 
 	if (getenv("UFTRACE_LIST_EVENT")) {
@@ -1984,6 +2141,7 @@ static void mcount_cleanup(void)
 #endif
 
 	mcount_filter_finish();
+	mcount_watch_finish();
 
 	if (SCRIPT_ENABLED && script_str)
 		script_finish();
@@ -1999,14 +2157,20 @@ static void mcount_cleanup(void)
  */
 #define UFTRACE_ALIAS(_func) void uftrace_##_func(void *, void *) __alias(_func)
 
+/* This is the historic startup routine for mcount but not used here. */
 void __visible_default __monstartup(unsigned long low, unsigned long high)
 {
 }
 
+/* This is the historic cleanup routine for mcount but not used here. */
 void __visible_default _mcleanup(void)
 {
 }
 
+/*
+ * This is a non-standard external function to work around some stack
+ * corruption problems in the past.  I hope we don't need it anymore.
+ */
 void __visible_default mcount_restore(void)
 {
 	struct mcount_thread_data *mtdp;
@@ -2018,6 +2182,10 @@ void __visible_default mcount_restore(void)
 	mcount_rstack_restore(mtdp);
 }
 
+/*
+ * This is a non-standard external function to work around some stack
+ * corruption problems in the past.  I hope we don't need it anymore.
+ */
 void __visible_default mcount_reset(void)
 {
 	struct mcount_thread_data *mtdp;
@@ -2026,9 +2194,13 @@ void __visible_default mcount_reset(void)
 	if (unlikely(check_thread_data(mtdp)))
 		return;
 
-	mcount_rstack_reset(mtdp);
+	mcount_rstack_rehook(mtdp);
 }
 
+/*
+ * External entry points for -finstrument-functions.  The alias was added to
+ * avoid calling them through PLT.
+ */
 void __visible_default __cyg_profile_func_enter(void *child, void *parent)
 {
 	cygprof_entry((unsigned long)parent, (unsigned long)child);
