@@ -73,7 +73,7 @@ void prepare_shmem_buffer(struct mcount_thread_data *mtdp)
 
 	shmem->nr_buf = 2;
 	shmem->max_buf = 2;
-	shmem->buffer = xcalloc(sizeof(*shmem->buffer), 2);
+	shmem->buffer = xcalloc(2, sizeof(*shmem->buffer));
 
 	for (idx = 0; idx < shmem->nr_buf; idx++) {
 		shmem->buffer[idx] = allocate_shmem_buffer(buf, sizeof(buf), tid, idx);
@@ -240,6 +240,10 @@ static struct mcount_event *get_event_pointer(void *base, unsigned idx)
 }
 
 #ifndef DISABLE_MCOUNT_FILTER
+/*
+ * These functions are for regular libmcount with filters.
+ */
+
 void *get_argbuf(struct mcount_thread_data *mtdp, struct mcount_ret_stack *rstack)
 {
 	ptrdiff_t idx = rstack - mtdp->rstack;
@@ -785,9 +789,43 @@ void save_watchpoint(struct mcount_thread_data *mtdp, struct mcount_ret_stack *r
 		}
 		mtdp->watch.cpu = cpu;
 	}
+
+	if (watchpoints & MCOUNT_WATCH_VAR) {
+		struct mcount_watchpoint_item *w;
+		unsigned long watch_data = 0;
+		struct mcount_event *event;
+
+		list_for_each_entry(w, &mtdp->watch.list, list) {
+			if (mtdp->nr_events >= MAX_EVENT)
+				continue;
+
+			/* check the data without lock first */
+			mcount_memcpy1(&watch_data, (void *)w->addr, w->size);
+			if (!memcmp(&watch_data, w->data, w->size))
+				continue;
+
+			/* make sure only one thread updates the watch data */
+			if (!mcount_watch_update(w->addr, &watch_data, w->size))
+				continue;
+
+			event = &mtdp->event[mtdp->nr_events++];
+
+			event->id = EVENT_ID_WATCH_VAR;
+			event->time = timestamp;
+			event->idx = rstack_idx;
+			event->dsize = sizeof(long) + w->size;
+
+			mcount_memcpy4(event->data, &w->addr, sizeof(long));
+			mcount_memcpy1(event->data + sizeof(long), &watch_data, w->size);
+		}
+	}
 }
 
 #else
+/*
+ * These are for fast libmcount libraries without filters.
+ */
+
 void *get_argbuf(struct mcount_thread_data *mtdp, struct mcount_ret_stack *rstack)
 {
 	return NULL;
@@ -863,7 +901,17 @@ static int record_event(struct mcount_thread_data *mtdp, struct mcount_event *ev
 
 	/*
 	 * instead of set bit fields, do the bit operations manually.
-	 * this would be good for both performance and portability.
+	 * this would be good for both performance and portability,
+	 * and should be equivalent to the following:
+	 *
+	 *	struct uftrace_record *data = curr_buf->data + curr_buf->size;
+	 *
+	 *	data->time   = event->time;
+	 *	data->type   = UFTRACE_EVENT;
+	 *	data->magic  = RECORD_MAGIC;
+	 *	data->more   = 0;
+	 *	data->depth  = 0;
+	 *	data->addr   = event->id;
 	 */
 	rec->data = UFTRACE_EVENT | RECORD_MAGIC << 3;
 	rec->data += (uint64_t)event->id << 16;
@@ -875,7 +923,7 @@ static int record_event(struct mcount_thread_data *mtdp, struct mcount_event *ev
 		rec->data += 4; /* set 'more' bit in uftrace_record */
 
 		*(uint16_t *)ptr = data_size;
-		memcpy(ptr + 2, event->data, data_size);
+		mcount_memcpy1(ptr + 2, event->data, data_size);
 	}
 
 	curr_buf->size += size;
@@ -941,19 +989,19 @@ static int record_ret_stack(struct mcount_thread_data *mtdp, enum uftrace_record
 	if (curr_buf == NULL)
 		return mtdp->shmem.done ? 0 : -1;
 
-#if 0
-	frstack = (void *)(curr_buf->data + curr_buf->size);
-
-	frstack->time   = timestamp;
-	frstack->type   = type;
-	frstack->magic  = RECORD_MAGIC;
-	frstack->more   = !!argbuf;
-	frstack->depth  = mrstack->depth;
-	frstack->addr   = mrstack->child_ip;
-#else
 	/*
 	 * instead of set bit fields, do the bit operations manually.
-	 * this would be good for both performance and portability.
+	 * this would be good for both performance and portability,
+	 * and should be equivalent to the following:
+	 *
+	 *	frstack = (void *)(curr_buf->data + curr_buf->size);
+	 *
+	 *	frstack->time   = timestamp;
+	 *	frstack->type   = type;
+	 *	frstack->magic  = RECORD_MAGIC;
+	 *	frstack->more   = !!argbuf;
+	 *	frstack->depth  = mrstack->depth;
+	 *	frstack->addr   = mrstack->child_ip;
 	 */
 	rec = type | RECORD_MAGIC << 3;
 	rec += argbuf ? 4 : 0;
@@ -963,7 +1011,6 @@ static int record_ret_stack(struct mcount_thread_data *mtdp, enum uftrace_record
 	buf = (void *)(curr_buf->data + curr_buf->size);
 	buf[0] = timestamp;
 	buf[1] = rec;
-#endif
 
 	curr_buf->size += sizeof(*frstack);
 	mrstack->flags |= MCOUNT_FL_WRITTEN;
@@ -1003,6 +1050,30 @@ static int record_ret_stack(struct mcount_thread_data *mtdp, enum uftrace_record
 	return 0;
 }
 
+/*
+ * For performance reasons and time filter, it doesn't record trace data one at
+ * a time.  Instead it usually writes the data when an EXIT record is ready so
+ * it needs to record ENTRY data in the current and may in the parent functions.
+ *
+ * For example, if it has a time filter for 1 usec.
+ *
+ * foo() {
+ *   bar() {
+ *     leaf1();   // takes 0.5 usec
+ *     leaf2();   // takes 1.2 usec
+ *
+ * Then it can start to record when leaf2 function returns (at this moment,
+ * mcount_ret_stack for leaf1 is gone) then it'd save the following records
+ * (unless ENTRY foo or bar is saved by an earlier child before leaf[12]).
+ *
+ *   ENTRY (foo)
+ *   ENTRY (bar)
+ *   ENTRY (leaf2)
+ *   EXIT  (leaf2)
+ *
+ * Then it adds MCOUNT_FL_WRITTEN flag to parent (foo and bar) so that they
+ * never be written anymore by other child function.
+ */
 int record_trace_data(struct mcount_thread_data *mtdp, struct mcount_ret_stack *mrstack,
 		      long *retval)
 {
