@@ -19,8 +19,11 @@
 extern struct uftrace_sym_info mcount_sym_info;
 
 struct dlopen_base_data {
+	const char *filename;
 	struct mcount_thread_data *mtdp;
+	struct uftrace_triggers_info *triggers;
 	uint64_t timestamp;
+	void *handle;
 };
 
 static void send_dlopen_msg(struct mcount_thread_data *mtdp, const char *sess_id,
@@ -56,20 +59,21 @@ static void send_dlopen_msg(struct mcount_thread_data *mtdp, const char *sess_id
 	};
 	int len = sizeof(msg) + msg.len;
 
-	if (pfd < 0)
+	if (mcount_pfd < 0)
 		return;
 
 	mcount_memcpy4(dlop.sid, sess_id, sizeof(dlop.sid));
 
-	if (writev(pfd, iov, 3) != len) {
+	if (writev(mcount_pfd, iov, 3) != len) {
 		if (!mcount_should_stop())
-			pr_err("write tid info failed");
+			pr_err("send dlopen msg failed");
 	}
 }
 
 static int dlopen_base_callback(struct dl_phdr_info *info, size_t size, void *arg)
 {
 	struct dlopen_base_data *data = arg;
+	struct uftrace_mmap *map;
 	char buf[PATH_MAX];
 	char *p;
 
@@ -77,6 +81,8 @@ static int dlopen_base_callback(struct dl_phdr_info *info, size_t size, void *ar
 		return 0;
 	if (!strcmp("linux-vdso.so.1", info->dlpi_name))
 		return 0;
+	if (!strstr(info->dlpi_name, data->filename))
+		return 0;
 
 	p = realpath(info->dlpi_name, buf);
 	if (p == NULL)
@@ -89,11 +95,41 @@ static int dlopen_base_callback(struct dl_phdr_info *info, size_t size, void *ar
 	send_dlopen_msg(data->mtdp, mcount_session_name(), data->timestamp, info->dlpi_addr,
 			info->dlpi_name);
 
-	mcount_dynamic_dlopen(&mcount_sym_info, info, p);
+	map = xzalloc(sizeof(*map) + strlen(p) + 1);
+	map->len = strlen(p);
+	strcpy(map->libname, p);
+	mcount_memcpy1(map->prot, "r-xp", 4);
+	map->handle = data->handle;
+
+	for (int i = 0; i < info->dlpi_phnum; i++) {
+		if (info->dlpi_phdr[i].p_type != PT_LOAD)
+			continue;
+
+		if (map->start == 0)
+			map->start = info->dlpi_phdr[i].p_vaddr + info->dlpi_addr;
+
+		if (info->dlpi_phdr[i].p_flags & PF_X) {
+			map->end = info->dlpi_phdr[i].p_vaddr + info->dlpi_addr;
+			map->end += info->dlpi_phdr[i].p_memsz;
+			break;
+		}
+	}
+
+	read_build_id(p, map->build_id, sizeof(map->build_id));
+	map->mod = load_module_symtab(&mcount_sym_info, p, map->build_id);
+
+	map->next = mcount_sym_info.maps;
+	write_memory_barrier();
+	mcount_sym_info.maps = map;
+
+	mcount_dynamic_dlopen(&mcount_sym_info, info, p, map);
+
+	data->triggers = mcount_trigger_init(&mcount_filter_setting);
+
 	return 0;
 }
 
-void mcount_rstack_reset_exception(struct mcount_thread_data *mtdp, unsigned long frame_addr)
+void mcount_rstack_rehook_exception(struct mcount_thread_data *mtdp, unsigned long frame_addr)
 {
 	int idx;
 	struct mcount_ret_stack *rstack;
@@ -148,7 +184,7 @@ void mcount_rstack_reset_exception(struct mcount_thread_data *mtdp, unsigned lon
 	mtdp->idx = idx + 1;
 	pr_dbg3("%s: exception returned to [%d]\n", __func__, mtdp->idx);
 
-	mcount_rstack_reset(mtdp);
+	mcount_rstack_rehook(mtdp);
 }
 
 static char **collect_uftrace_envp(void)
@@ -197,7 +233,7 @@ static char **collect_uftrace_envp(void)
 			n++;
 	}
 
-	envp = xcalloc(sizeof(*envp), n + 2);
+	envp = xcalloc(n + 2, sizeof(*envp));
 
 	for (i = k = 0; i < ARRAY_SIZE(uftrace_env); i++) {
 		char *env_str;
@@ -232,7 +268,7 @@ static char **merge_envp(char *const *env1, char **env2)
 	n += count_envp(env1);
 	n += count_envp(env2);
 
-	envp = xcalloc(sizeof(*envp), n + 1);
+	envp = xcalloc(n + 1, sizeof(*envp));
 
 	n = 0;
 	for (i = 0; env1 && env1[i]; i++)
@@ -253,6 +289,7 @@ static void *(*real_cxa_begin_catch)(void *exc);
 static void (*real_cxa_end_catch)(void);
 static void (*real_cxa_guard_abort)(void *guard_obj);
 static void *(*real_dlopen)(const char *filename, int flags);
+static int (*real_dlclose)(void *handle);
 static __noreturn void (*real_pthread_exit)(void *retval);
 static void (*real_unwind_resume)(void *exc);
 static int (*real_posix_spawn)(pid_t *pid, const char *path,
@@ -267,6 +304,7 @@ static int (*real_posix_spawnp)(pid_t *pid, const char *file,
 static int (*real_execve)(const char *path, char *const argv[], char *const envp[]);
 static int (*real_execvpe)(const char *file, char *const argv[], char *const envp[]);
 static int (*real_fexecve)(int fd, char *const argv[], char *const envp[]);
+static int (*real_close)(int fd);
 
 void mcount_hook_functions(void)
 {
@@ -277,6 +315,7 @@ void mcount_hook_functions(void)
 	real_cxa_end_catch = dlsym(RTLD_NEXT, "__cxa_end_catch");
 	real_cxa_guard_abort = dlsym(RTLD_NEXT, "__cxa_guard_abort");
 	real_dlopen = dlsym(RTLD_NEXT, "dlopen");
+	real_dlclose = dlsym(RTLD_NEXT, "dlclose");
 	real_pthread_exit = dlsym(RTLD_NEXT, "pthread_exit");
 	real_unwind_resume = dlsym(RTLD_NEXT, "_Unwind_Resume");
 	real_posix_spawn = dlsym(RTLD_NEXT, "posix_spawn");
@@ -284,6 +323,7 @@ void mcount_hook_functions(void)
 	real_execve = dlsym(RTLD_NEXT, "execve");
 	real_execvpe = dlsym(RTLD_NEXT, "execvpe");
 	real_fexecve = dlsym(RTLD_NEXT, "fexecve");
+	real_close = dlsym(RTLD_NEXT, "close");
 }
 
 __visible_default int backtrace(void **buffer, int sz)
@@ -303,7 +343,7 @@ __visible_default int backtrace(void **buffer, int sz)
 	ret = real_backtrace(buffer, sz);
 
 	if (!check_thread_data(mtdp))
-		mcount_rstack_reset(mtdp);
+		mcount_rstack_rehook(mtdp);
 
 	return ret;
 }
@@ -324,7 +364,7 @@ __visible_default void __cxa_throw(void *exception, void *type, void *dest)
 		/*
 		 * restore return addresses so that it can unwind stack
 		 * frames safely during the exception handling.
-		 * It pairs to mcount_rstack_reset_exception().
+		 * It pairs to mcount_rstack_rehook_exception().
 		 */
 		mcount_rstack_restore(mtdp);
 	}
@@ -348,7 +388,7 @@ __visible_default void __cxa_rethrow(void)
 		/*
 		 * restore return addresses so that it can unwind stack
 		 * frames safely during the exception handling.
-		 * It pairs to mcount_rstack_reset_exception()
+		 * It pairs to mcount_rstack_rehook_exception()
 		 */
 		mcount_rstack_restore(mtdp);
 	}
@@ -372,7 +412,7 @@ __visible_default void _Unwind_Resume(void *exception)
 		/*
 		 * restore return addresses so that it can unwind stack
 		 * frames safely during the exception handling.
-		 * It pairs to mcount_rstack_reset_exception().
+		 * It pairs to mcount_rstack_rehook_exception().
 		 */
 		mcount_rstack_restore(mtdp);
 	}
@@ -402,7 +442,7 @@ __visible_default void *__cxa_begin_catch(void *exception)
 		if (frame_addr < (unsigned long)frame_ptr)
 			frame_addr = (unsigned long)frame_ptr;
 
-		mcount_rstack_reset_exception(mtdp, frame_addr);
+		mcount_rstack_rehook_exception(mtdp, frame_addr);
 		mtdp->in_exception = false;
 		pr_dbg2("%s: exception caught begin on [%d]\n", __func__, mtdp->idx);
 	}
@@ -441,7 +481,7 @@ __visible_default void __cxa_guard_abort(void *guard_obj)
 		if (frame_addr < (unsigned long)frame_ptr)
 			frame_addr = (unsigned long)frame_ptr;
 
-		mcount_rstack_reset_exception(mtdp, frame_addr);
+		mcount_rstack_rehook_exception(mtdp, frame_addr);
 		mtdp->in_exception = false;
 
 		/*
@@ -463,6 +503,7 @@ __visible_default void *dlopen(const char *filename, int flags)
 	struct mcount_thread_data *mtdp;
 	struct dlopen_base_data data = {
 		.timestamp = mcount_gettime(),
+		.filename = filename,
 	};
 	void *ret;
 
@@ -493,12 +534,53 @@ __visible_default void *dlopen(const char *filename, int flags)
 	}
 
 	data.mtdp = mtdp;
+	data.handle = ret;
 	dl_iterate_phdr(dlopen_base_callback, &data);
 
+	if (data.triggers)
+		swap_triggers(&mcount_triggers, data.triggers);
+
 	mcount_unguard_recursion(mtdp);
 	return ret;
 }
 
+__visible_default int dlclose(void *handle)
+{
+	struct mcount_thread_data *mtdp;
+	struct uftrace_mmap *map;
+	int ret;
+
+	if (unlikely(real_dlopen == NULL))
+		mcount_hook_functions();
+
+	ret = real_dlclose(handle);
+
+	mtdp = get_thread_data();
+	if (unlikely(check_thread_data(mtdp))) {
+		mtdp = mcount_prepare();
+		if (mtdp == NULL)
+			return ret;
+	}
+	else {
+		if (!mcount_guard_recursion(mtdp))
+			return ret;
+	}
+
+	for_each_map(&mcount_sym_info, map) {
+		if (map->mod == NULL)
+			continue;
+
+		if (map->handle == handle) {
+			map->mod = NULL;
+			break;
+		}
+	}
+
+	mcount_unguard_recursion(mtdp);
+
+	return ret;
+}
+
 __visible_default __noreturn void pthread_exit(void *retval)
 {
 	struct mcount_thread_data *mtdp;
@@ -608,6 +690,18 @@ __visible_default int fexecve(int fd, char *const argv[], char *const envp[])
 	return real_fexecve(fd, argv, new_envp);
 }
 
+__visible_default int close(int fd)
+{
+	if (unlikely(real_close == NULL))
+		mcount_hook_functions();
+
+	if (unlikely(fd == mcount_pfd)) {
+		return 0;
+	}
+
+	return real_close(fd);
+}
+
 #ifdef UNIT_TEST
 
 TEST_CASE(mcount_wrap_dlopen)
