@@ -1,4 +1,4 @@
-static volatile int acount;
+static volatile int account;
 
 extern volatile int tcount;
 extern void t1(void);
@@ -7,18 +7,17 @@ extern void t2(void);
 static void __attribute__((noinline)) a1(void)
 {
 	t1();
-	acount++;
+	account++;
 }
 
 void __attribute__((noinline)) a2(void)
 {
 	t2();
-	acount++;
+	account++;
 }
 
 int main(void)
 {
 	a1();
-	return acount + tcount;
+	return account + tcount;
 }
-
