@@ -325,7 +325,7 @@ class TestBase:
             # ignore result of remaining functions which follows a blank line
             if ln.strip() == '':
                 break
-            pid_patt = re.compile('[^[]+\[ *(\d+)\] |')
+            pid_patt = re.compile(r'[^[]+\[ *(\d+)\] |')
             m = pid_patt.match(ln)
             try:
                 pid = int(m.group(1))
@@ -463,8 +463,6 @@ class TestBase:
         except Exception:
             return ''
         for ln in o['traceEvents']:
-            if ln['name'].startswith('__'):
-                continue
             if ln['ph'] == "M":
                 if ln['name'] == "process_name" or ln['name'] == "thread_name":
                     args = ln['args']
@@ -493,7 +491,7 @@ class TestBase:
                 break
 
             m = re.match( r'\s+(?P<start_depth>\d+)_(?P<start_id>\d+)\[\"(?P<start_name>\S+)\"\]\s+'
-                + '-->\|(?P<call_num>\d+)\|\s+(?P<end_depth>\d+)_(?P<end_id>\d+)\[\"(?P<end_name>\S+)\"\];', ln)
+                + r'-->\|(?P<call_num>\d+)\|\s+(?P<end_depth>\d+)_(?P<end_id>\d+)\[\"(?P<end_name>\S+)\"\];', ln)
             if m:
                 result.append("%s_%s_%s %s> %s_%s_%s" % (m.group('start_depth'), m.group('start_id'), m.group('start_name'),
                 m.group('call_num'), m.group('end_depth'), m.group('end_id'), m.group('end_name')))
@@ -656,7 +654,8 @@ class TestBase:
             if ret == 2:
                 return TestBase.TEST_ABNORMAL_EXIT, ''
             if diff:
-                dif = "%s: %s returns %d\n" % (name, cflags, ret)
+                compiler = self.supported_lang[self.lang]['cc']
+                dif = "%s: %s %s returns %d\n" % (name, compiler, cflags, ret)
             return TestBase.TEST_NONZERO_RETURN, dif
 
         self.pr_debug("=========== %s ===========\n%s" % ("original", result_origin))
@@ -684,7 +683,7 @@ class TestBase:
                 f.write(result_tested + '\n')
                 f.close()
 
-                compiler = self.supported_lang['C']['cc']
+                compiler = self.supported_lang[self.lang]['cc']
                 dif = "%s: diff result of %s %s\n" % (name, compiler, cflags)
                 try:
                     p = sp.Popen(['colordiff', '-U1', 'expect', 'result'], stdout=sp.PIPE)
@@ -802,7 +801,7 @@ def run_single_case(case, flags, opts, arg, compilers):
                 tc.set_keep(arg.keep)
                 tc.set_compiler(compiler)
 
-                cflags = ' '.join(["-" + flag, "-" + opt])
+                cflags = ' '.join(["-" + flag, "-" + opt, tc.cflags])
                 # add -fno-ipa-sra to prevent function renames like foo.isra.0
                 # this is available on GCC only
                 if compiler == 'gcc':
