@@ -1,5 +1,5 @@
-#include <stdio.h>
 #include <stdarg.h>
+#include <stdio.h>
 
 int variadic(const char *fmt, ...)
 {
@@ -16,7 +16,7 @@ int variadic(const char *fmt, ...)
 
 int main(void)
 {
-	variadic("print %c %s %d %ld %lu %lld %f",
-		 'a', "hello", 100, 1234L, 5678UL, 9876543210ULL, 3.141592);
+	variadic("print %c %s %d %ld %lu %lld %f", 'a', "hello", 100, 1234L, 5678UL, 9876543210ULL,
+		 3.141592);
 	return 0;
 }
