@@ -98,6 +98,7 @@ enum uftrace_info_bits {
 	RECORD_DATE_BIT,
 	PATTERN_TYPE_BIT,
 	VERSION_BIT,
+	UTC_OFFSET_BIT,
 
 	INFO_BIT_MAX,
 
@@ -116,6 +117,7 @@ enum uftrace_info_bits {
 	RECORD_DATE = (1U << RECORD_DATE_BIT),
 	PATTERN_TYPE = (1U << PATTERN_TYPE_BIT),
 	VERSION = (1U << VERSION_BIT),
+	UTC_OFFSET = (1U << UTC_OFFSET_BIT),
 };
 
 struct uftrace_info {
@@ -142,6 +144,7 @@ struct uftrace_info {
 	double utime;
 	char *record_date;
 	char *elapsed_time;
+	char *utc_offset;
 	long vctxsw;
 	long ictxsw;
 	long maxrss;
@@ -354,6 +357,7 @@ void put_libmcount_path(char *libpath);
 #define SESSION_ID_LEN 16
 #define TASK_COMM_LEN 16
 #define TASK_COMM_LAST (TASK_COMM_LEN - 1)
+#define TASK_ID_LEN 7
 
 struct uftrace_session {
 	struct rb_node node;
@@ -379,6 +383,7 @@ struct uftrace_dlopen_list {
 	uint64_t time;
 	unsigned long base;
 	struct uftrace_module *mod;
+	struct rb_root filters;
 };
 
 struct uftrace_task {
@@ -482,9 +487,15 @@ void delete_session_map(struct uftrace_sym_info *sinfo);
 void update_session_map(const char *filename);
 struct uftrace_session *get_session_from_sid(struct uftrace_session_link *sess, char sid[]);
 void session_add_dlopen(struct uftrace_session *sess, uint64_t timestamp, unsigned long base_addr,
-			const char *libname);
+			const char *libname, bool needs_srcline);
+void session_setup_dlopen_argspec(struct uftrace_session *sess,
+				  struct uftrace_filter_setting *setting, bool is_retval);
+struct uftrace_dlopen_list *session_find_dlopen(struct uftrace_session *sess, uint64_t timestamp,
+						unsigned long addr);
 struct uftrace_symbol *session_find_dlsym(struct uftrace_session *sess, uint64_t timestamp,
 					  unsigned long addr);
+struct uftrace_filter *session_find_filter(struct uftrace_session *sess, struct uftrace_record *rec,
+					   struct uftrace_trigger *tr);
 void delete_sessions(struct uftrace_session_link *sess);
 
 struct uftrace_record;
@@ -612,6 +623,7 @@ enum uftrace_event_id {
 	EVENT_ID_READ_PMU_BRANCH,
 	EVENT_ID_DIFF_PMU_BRANCH,
 	EVENT_ID_WATCH_CPU,
+	EVENT_ID_WATCH_VAR,
 
 	/* supported perf events */
 	EVENT_ID_PERF = 200000U,
@@ -636,6 +648,16 @@ struct uftrace_event {
 	char *event;
 };
 
+struct uftrace_watch_event {
+	union {
+		int cpu;
+		struct {
+			uint64_t addr;
+			uint64_t data;
+		} var;
+	};
+};
+
 #define HTML_HEADER                                                                                \
 	"<html>\n"                                                                                 \
 	"<head></head>\n"                                                                          \
