@@ -40,13 +40,18 @@ enum display_field_id {
 	REPORT_F_TOTAL_TIME_AVG,
 	REPORT_F_TOTAL_TIME_MIN,
 	REPORT_F_TOTAL_TIME_MAX,
+
 	REPORT_F_SELF_TIME,
 	REPORT_F_SELF_TIME_AVG,
 	REPORT_F_SELF_TIME_MIN,
 	REPORT_F_SELF_TIME_MAX,
+
 	REPORT_F_CALL,
 	REPORT_F_SIZE,
 
+	REPORT_F_TOTAL_TIME_STDV,
+	REPORT_F_SELF_TIME_STDV,
+
 	REPORT_F_TASK_TOTAL_TIME = 0,
 	REPORT_F_TASK_SELF_TIME,
 	REPORT_F_TASK_TID,
