@@ -448,6 +448,7 @@ static int build_arg_spec(struct uftrace_session *s, void *arg)
 		s->filters = triggers.root;
 	}
 
+	session_setup_dlopen_argspec(s, setting, false);
 	return 0;
 }
 
@@ -463,6 +464,7 @@ static int build_ret_spec(struct uftrace_session *s, void *arg)
 		s->filters = triggers.root;
 	}
 
+	session_setup_dlopen_argspec(s, setting, true);
 	return 0;
 }
 
@@ -610,8 +612,8 @@ int fstack_entry(struct uftrace_task_reader *task, struct uftrace_record *rstack
 	if (fstack == NULL)
 		return -1;
 
-	pr_dbg2("ENTRY: [%5d] stack: %d, depth: %d, disp: %d, I: %d, O: %d, D: %d, flags = %lx %s\n",
-		task->tid, task->stack_count - 1, rstack->depth, task->display_depth,
+	pr_dbg2("ENTRY: [%*d] stack: %d, depth: %d, disp: %d, I: %d, O: %d, D: %d, flags = %lx %s\n",
+		TASK_ID_LEN, task->tid, task->stack_count - 1, rstack->depth, task->display_depth,
 		task->filter.in_count, task->filter.out_count, task->filter.depth, fstack->flags,
 		rstack->more ? "more" : "");
 
@@ -740,8 +742,8 @@ void fstack_exit(struct uftrace_task_reader *task)
 	if (fstack == NULL)
 		return;
 
-	pr_dbg2("EXIT : [%5d] stack: %d, depth: %d, disp: %d, I: %d, O: %d, D: %d, flags = %lx\n",
-		task->tid, task->stack_count, task->rstack->depth, task->display_depth,
+	pr_dbg2("EXIT : [%*d] stack: %d, depth: %d, disp: %d, I: %d, O: %d, D: %d, flags = %lx\n",
+		TASK_ID_LEN, task->tid, task->stack_count, task->rstack->depth, task->display_depth,
 		task->filter.in_count, task->filter.out_count, task->filter.depth, fstack->flags);
 
 	if (fstack->flags & FSTACK_FL_FILTERED)
@@ -1021,10 +1023,10 @@ bool fstack_check_filter(struct uftrace_task_reader *task)
 			if (fstack == NULL)
 				return false;
 
-			pr_dbg2("SCHED: [%5d] stack: %d, depth: %d, disp: %d, I: %d, O: %d, D: %d, %s\n",
-				task->tid, idx, fstack->orig_depth, task->display_depth,
-				task->filter.in_count, task->filter.out_count, task->filter.depth,
-				sched);
+			pr_dbg2("SCHED: [%*d] stack: %d, depth: %d, disp: %d, I: %d, O: %d, D: %d, %s\n",
+				TASK_ID_LEN, task->tid, idx, fstack->orig_depth,
+				task->display_depth, task->filter.in_count, task->filter.out_count,
+				task->filter.depth, sched);
 		}
 	}
 
@@ -1345,7 +1347,7 @@ int read_task_args(struct uftrace_task_reader *task, struct uftrace_record *rsta
 		return -1;
 	}
 
-	fl = uftrace_match_filter(rstack->addr, &sess->filters, &tr);
+	fl = session_find_filter(sess, rstack, &tr);
 	if (fl == NULL) {
 		pr_dbg("cannot find filter: %lx\n", rstack->addr);
 		return -1;
@@ -1388,6 +1390,37 @@ static int read_task_event_size(struct uftrace_task_reader *task, void *buf, siz
 	return 0;
 }
 
+static int read_task_watch_event(struct uftrace_task_reader *task,
+				 struct uftrace_watch_event *watch, uint16_t *plen)
+{
+	uint16_t len;
+
+	memset(&watch->var, 0, sizeof(watch->var));
+
+	if (fread(&len, sizeof(len), 1, task->fp) != 1)
+		return -1;
+
+	*plen = len;
+
+	if (data_is_lp64(task->h)) {
+		if (fread(&watch->var.addr, 8, 1, task->fp) != 1)
+			return -1;
+
+		len -= 8;
+	}
+	else {
+		if (fread(&watch->var.addr, 4, 1, task->fp) != 1)
+			return -1;
+
+		len -= 4;
+	}
+
+	if (fread(&watch->var.data, len, 1, task->fp) != 1)
+		return -1;
+
+	return 0;
+}
+
 static void save_task_event(struct uftrace_task_reader *task, void *buf, size_t buflen)
 {
 	int rem;
@@ -1413,8 +1446,9 @@ int read_task_event(struct uftrace_task_reader *task, struct uftrace_record *rec
 		struct uftrace_pmu_cycle cycle;
 		struct uftrace_pmu_cache cache;
 		struct uftrace_pmu_branch branch;
-		int cpu;
+		struct uftrace_watch_event watch;
 	} u;
+	uint16_t len;
 
 	switch (rec->addr) {
 	case EVENT_ID_READ_PROC_STATM:
@@ -1484,12 +1518,18 @@ int read_task_event(struct uftrace_task_reader *task, struct uftrace_record *rec
 		break;
 
 	case EVENT_ID_WATCH_CPU:
-		if (read_task_event_size(task, &u.cpu, sizeof(u.cpu)) < 0)
+		if (read_task_event_size(task, &u.watch.cpu, sizeof(u.watch.cpu)) < 0)
 			return -1;
 		if (task->h->needs_byte_swap)
-			u.cpu = bswap_32(u.cpu);
+			u.watch.cpu = bswap_32(u.watch.cpu);
 
-		save_task_event(task, &u.cpu, sizeof(u.cpu));
+		save_task_event(task, &u.watch.cpu, sizeof(u.watch.cpu));
+		break;
+
+	case EVENT_ID_WATCH_VAR:
+		if (read_task_watch_event(task, &u.watch, &len) < 0)
+			return -1;
+		save_task_event(task, &u.watch, len);
 		break;
 
 	default:
@@ -2237,7 +2277,7 @@ static void adjust_rstack_after_schedule(struct uftrace_data *handle,
 			task->timestamp_next -= prev_fstack->total_time;
 		}
 
-		pr_dbg3("task[%6d] delay next record after schedule\n", task->tid);
+		pr_dbg3("task[%*d] delay next record after schedule\n", TASK_ID_LEN, task->tid);
 		next_rec->time = ~0ULL;
 		return;
 	}
@@ -2253,7 +2293,7 @@ static void adjust_rstack_after_schedule(struct uftrace_data *handle,
 	/* save it in case it's overwritten by subsequent schedule */
 	task->timestamp_estimate = next_rec->time;
 
-	pr_dbg3("task[%6d] estimate next record after schedule\n", task->tid);
+	pr_dbg3("task[%*d] estimate next record after schedule\n", TASK_ID_LEN, task->tid);
 }
 
 static int __read_rstack(struct uftrace_data *handle, struct uftrace_task_reader **taskp,
